import { Component, OnInit, ViewChild, ElementRef, Inject, Input } from '@angular/core';
import { DomSanitizer} from '@angular/platform-browser';
import { Renderer2 } from '@angular/core';
//import { MatDialogRef,MAT_DIALOG_DATA } from '@angular/material';
import { TesService } from '../tes.service';
import { timer, Subscription} from 'rxjs';
import { BehaviorSubject } from 'rxjs';
import { Identity } from '../identity';
import { Strudelapp } from '../strudelapp';
import { BatchInterface} from '../batchinterface';


@Component({
  selector: 'app-launch-dialog',
  templateUrl: './launch-dialog.component.html',
  styleUrls: ['./launch-dialog.component.css']
})
export class LaunchDialogComponent implements OnInit {
  @Input() identity: Identity;
  @Input() appSubject: BehaviorSubject<Strudelapp>;
  app: Strudelapp;
  batchcmdurl: string;
  appconfigurl: string;
  batchcmdsafeurl: any;
  appconfigsafeurl: any;
  rmListen: any;
  @ViewChild('launchbtn', { read: ElementRef, static: false }) launchbtn: ElementRef;
  @ViewChild('batchbuilderiframe', { read: ElementRef, static: false }) batchbuilderiframe: ElementRef;
  setFocus: Boolean;
  subscriptions: Subscription[];
  width: number;
  height: number;
  public appData: any;
  public submitcmd: string;

  //constructor( public dialogRef: MatDialogRef<LaunchDialogComponent>,
  constructor( 
    private domSanitizer: DomSanitizer,
    private renderer: Renderer2,
    private tesService: TesService,
  ) {
    this.submitcmd = null;
    this.appData = null
    this.subscriptions = [];

  }

  ngOnInit() {
    console.log('creating launch-dialog');
    this.rmListen = this.renderer.listen('window','message',(event) => this.receiveMessage(event));
    this.setFocus = false;
    this.subscriptions.push(this.appSubject.subscribe(() => { this.setReady(false); this.app = this.appSubject.value; this.updateURLs()}));
    
  }

  updateURLs() {
    let theme = "";
    theme=localStorage.getItem('strudel-theme');
    if (theme == null) {
      theme = 'strudel-theme-light';
    }
    if (this.identity !== null && this.app !== null) {
      this.batchcmdurl = this.identity.site.url+"/"+this.app.name+"?theme="+theme;
    }
    this.batchcmdsafeurl = this.domSanitizer.bypassSecurityTrustResourceUrl(this.batchcmdurl);
    if (this.app !== null) {
      this.appconfigurl = this.app.url+"?theme="+theme;
    }
    if (!(this.appconfigurl == null)) {
      this.appconfigsafeurl = this.domSanitizer.bypassSecurityTrustResourceUrl(this.appconfigurl);
    }
    console.log('updatedURLs');
  }

  ngOnDestroy() {
    this.rmListen();
    for (let s of this.subscriptions) {
      s.unsubscribe();
    }
  }


  setReady(ready) {
      if (this.launchbtn === undefined) {
          return
      }
      if (ready && !(this.setFocus)) {
        this.launchbtn.nativeElement._disabled = false;
        this.launchbtn.nativeElement.focus();
        this.setFocus = true;
      } else {
        this.launchbtn.nativeElement._disabled = true;
          console.log('launch is still disabled');
      }
  }


  receiveMessage(event) {
      if (event.data['batchcmd'] !== undefined)  {
        this.submitcmd = event.data['batchcmd'];
        console.log('recieved a submit cmd');
        console.log('recieveMessage'+this.submitcmd);
      }
      if (event.data['appData'] !== undefined)  {
        this.appData = event.data['appData'];
      }
      if (event.data['resize'] !== undefined) {
        console.log('setting iframe size');
        this.height = event.data['resize']['height'];
        this.width = event.data['resize']['width'];
        console.log(this.height);
      }
  }

  launch() {
    // tell the batchbuilder iframe that lanch was selected
    // This is the queue to save the values as defaults for next launch
    this.batchbuilderiframe.nativeElement.contentWindow.postMessage('launch selected',"*");
    let bi = new BatchInterface();
    bi.submitcmd = this.submitcmd;
    if (this.appData != null){ 
      let appparams = this.appData;
      this.tesService.submit(this.app,this.identity,bi,appparams);
    } else {
        this.tesService.submit(this.app,this.identity,bi);
    }
  }

  clearInfo() {
      this.appData = null;
      this.submitcmd = null;
  }

  canILaunch() {
    if (this.submitcmd != null) {
        if (this.app.url == null || this.appData != null ) {
            this.setReady(true);
            return;
        }
    }
    this.setReady(false);
  }
}
