import { Component, OnInit } from '@angular/core';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA, MatDialogModule, MatDialogConfig } from '@angular/material';
import {OverlayContainer} from '@angular/cdk/overlay';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Subscription } from 'rxjs';
import { Router, NavigationStart } from '@angular/router';

import {Strudelapp} from '../strudelapp';
import { StrudelappsService } from '../strudelapps.service';
import { TesService } from '../tes.service';
import {BackendSelectionService } from '../backend-selection.service';
import { AuthorisationService } from '../authorisation.service';
import { Identity } from '../identity';
import { Computesite } from '../computesite';
import { LogoutdialogComponent } from '../logoutdialog/logoutdialog.component';
import { SshAuthzServer } from '../identity';
import { ComputesitesService } from '../computesites.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit {
  public strudelapps: Strudelapp[];

  public app: Strudelapp;
  public authorised: boolean;
  public identity: Identity;

  public identities: Identity[];
  public sshauthzservers: SshAuthzServer[];
  private launchwindow: any;
  private launchwindowWatcher: any;
  public selectedApiServer: any;
  private file: any;
  private config: any;
  public theme: any;
  public themes: any[] = [ {'name':'Light','value':'strudel-light-theme'},
                   {'name': 'Dark','value':'strudel-dark-theme'}]
  public selected: any;
  private subscriptions: Subscription[];


  constructor( public dialog: MatDialog,
                public tesService: TesService,
                public backendSelectionService: BackendSelectionService,
                public authService: AuthorisationService,
                public computeSitesService: ComputesitesService,
                public overlayContainer: OverlayContainer,
                private router: Router,
                ) {


      this.subscriptions = []
      this.subscriptions.push(this.authService.sshAuthzServers.subscribe(o => {this.updateSshAuthZServers(o)}));
       
  }
    
  updateSshAuthZServers(o) {
    this.sshauthzservers = o;
  }

  ngOnInit() {
    //this.subscriptions.push(this.computeSitesService.appidentities.subscribe(o => this.navLaunch(o)));
    let navlaunch = localStorage.getItem('strudel-navlaunch')
    if (navlaunch == 'true') {
      localStorage.setItem('strudel-navlaunch',null)
      this.subscriptions.push(this.computeSitesService.appidentities.subscribe(o => this.navLaunch(o)));
    }
  }

  ngOnDestroy() {
    for (let s of this.subscriptions) {
      s.unsubscribe();
    }
  }

  navLaunch(o) {
    if (this.computeSitesService.appidentities.value.length > 0) {
      this.router.navigate(['/launch']);
    }
  }


  login () {
    localStorage.setItem('strudel-navlaunch','true');
    this.authService.login(this.selected);
  }

  signup(sshauthzserver) {
      window.open(sshauthzserver.signup);
  }

  loadConfig(event) {
      const reader = new FileReader();
      reader.onload = (e: any) => {
          try {
            this.config = JSON.parse(e.target.result);
              
              if ('computesites' in this.config) {
                  this.computeSitesService.storeLocalComputeSites(this.config['computesites'])
              }
              if ('apps' in this.config) {
                  this.computeSitesService.storeLocalStrudelApps(this.config['sitename'],this.config['apps']);
              }
              if ('authz' in this.config) {
                  this.authService.storeLocalAuthZ(this.config['authz'])
              }

          } catch {
              console.log('local config file is invalid')
          }
      };
      reader.readAsText(event.target.files[0]);
  }


  resetConfig() {
      this.computeSitesService.removeLocalStrudelApps();
      this.computeSitesService.removeLocalComputeSites();
      this.authService.removeLocalAuthZ()
  }

  setTheme(v)  {
    console.log('Theme selected');
    console.log(v);
    let theme = v;
    console.log(this.overlayContainer.getContainerElement().classList);
    let classList = document.querySelector('app-root').classList
    console.log('this theme');
    console.log(this.theme);
    if (theme == 'strudel-light-theme') {
      classList.remove('strudel-dark-theme');
      classList.add('strudel-light-theme');
    }
    if (theme == 'strudel-dark-theme') {
      classList.remove('strudel-light-theme');
      classList.add('strudel-dark-theme');
    }
    console.log('setting localStorage theme',v);
}

  selectTheme(event) {
    localStorage.setItem('strudel-theme',event.value);
  }
}

